@@ -79,13 +79,12 @@ const uint64_t XMM = _XMM | _XMM2;
 const uint64_t YMM = _YMM | _YMM2;
 const uint64_t K = 1ULL << 43;
 const uint64_t _ZMM = 1ULL << 44;
-const uint64_t _ZMM2 = 1ULL << 45;
 #ifdef XBYAK64
+const uint64_t _ZMM2 = 1ULL << 45;
 const uint64_t ZMM = _ZMM | _ZMM2;
 const uint64_t _YMM3 = 1ULL << 46;
 #else
 const uint64_t ZMM = _ZMM;
-const uint64_t _YMM3 = 0;
 #endif
 const uint64_t K2 = 1ULL << 47;
 const uint64_t ZMM_SAE = 1ULL << 48;
@@ -98,10 +97,6 @@ const uint64_t XMM_SAE = 1ULL << 51;
 const uint64_t XMM_KZ = 1ULL << 52;
 const uint64_t YMM_KZ = 1ULL << 53;
 const uint64_t ZMM_KZ = 1ULL << 54;
-#else
-const uint64_t XMM_KZ = 0;
-const uint64_t YMM_KZ = 0;
-const uint64_t ZMM_KZ = 0;
 #endif
 const uint64_t MEM_K = 1ULL << 55;
 const uint64_t M_1to2 = 1ULL << 56;
@@ -302,7 +297,7 @@ class Test {
 		case MEM64:
 			return "qword [eax+ecx*8]";
 		case MEM_ONLY_DISP:
-			return isXbyak_ ? "ptr[(void*)0x123]" : "[0x123]";
+			return isXbyak_ ? "ptr[(void*)0x123]" : "[abs 0x123]";
 		case _REG16: // not ax
 			{
 				static const char Reg16Tbl[][4] = {
@@ -533,6 +528,7 @@ class Test {
 			"nop",
 
 			"sahf",
+			"serialize",
 			"stc",
 			"std",
 			"sti",
@@ -557,6 +553,7 @@ class Test {
 			"wbinvd",
 			"wrmsr",
 			"xlatb",
+			"xend",
 
 			"popf",
 			"pushf",
@@ -637,6 +634,7 @@ class Test {
 				"fstsw",
 				"fnstsw",
 				"fxrstor",
+				"clwb",
 			};
 			for (size_t i = 0; i < NUM_OF_ARRAY(memTbl); i++) {
 				put(memTbl[i], MEM);
@@ -680,8 +678,8 @@ class Test {
 #endif
 
 #ifdef XBYAK64
-		put("jmp", "ptr[(void*)0x12345678]", "[0x12345678]");
-		put("call", "ptr[(void*)0x12345678]", "[0x12345678]");
+		put("jmp", "ptr[(void*)0x12345678]", "[abs 0x12345678]");
+		put("call", "ptr[(void*)0x12345678]", "[abs 0x12345678]");
 #ifdef USE_YASM
 		put("jmp", "ptr[rip + 0x12345678]", "[rip+0x12345678]");
 		put("call", "ptr[rip + 0x12345678]", "[rip+0x12345678]");
@@ -704,11 +702,11 @@ class Test {
 	void putFarJmp() const
 	{
 #ifdef XBYAK64
-		put("jmp", "word[rax],T_FAR", "far word [rax]");
+//		put("jmp", "word[rax],T_FAR", "far word [rax]");
 		put("jmp", "dword[rax],T_FAR", "far dword [rax]");
 		put("jmp", "qword[rax],T_FAR", "far qword [rax]");
 
-		put("call", "word[rax],T_FAR", "far word [rax]");
+//		put("call", "word[rax],T_FAR", "far word [rax]");
 		put("call", "dword[rax],T_FAR", "far dword [rax]");
 		put("call", "qword[rax],T_FAR", "far qword [rax]");
 #else
@@ -1016,9 +1014,7 @@ class Test {
 	}
 	void putCmov() const
 	{
-		const struct {
-			const char *s;
-		} tbl[] = {
+		const char tbl[][4] = {
 			"o",
 			"no",
 			"b",
@@ -1050,15 +1046,22 @@ class Test {
 			"nle",
 			"g",
 		};
+#if defined(__GNUC__) && !defined(__clang__)
+	#pragma GCC diagnostic push
+	#pragma GCC diagnostic ignored "-Wformat-truncation" // wrong detection
+#endif
 		for (size_t i = 0; i < NUM_OF_ARRAY(tbl); i++) {
 			char buf[32];
-			snprintf(buf, sizeof(buf), "cmov%s", tbl[i].s);
+			snprintf(buf, sizeof(buf), "cmov%s", tbl[i]);
 			put(buf, REG16, REG16|MEM);
 			put(buf, REG32, REG32|MEM);
 			put(buf, REG64, REG64|MEM);
-			snprintf(buf, sizeof(buf), "set%s", tbl[i].s);
+			snprintf(buf, sizeof(buf), "set%s", tbl[i]);
 			put(buf, REG8|REG8_3|MEM);
 		}
+#if defined(__GNUC__) && !defined(__clang__)
+	#pragma GCC diagnostic pop
+#endif
 	}
 	void putReg1() const
 	{
@@ -1091,6 +1094,7 @@ class Test {
 				put(p, REG64|RAX, "0x12345678", "0x12345678");
 				put(p, REG64|RAX, "192", "192");
 				put(p, REG64|RAX, "0x1234", "0x1234");
+				put(p, AX, "0x8000", "0x8000");
 				put(p, REG32|EAX, IMM8|IMM32|NEG8);
 				put(p, REG16|AX, IMM8|IMM16|NEG8|NEG16);
 				put(p, REG8|REG8_3|AL, IMM|NEG8);
@@ -1161,7 +1165,6 @@ class Test {
 			reduce 2-byte stack, so I can't support it
 		*/
 
-		put("push", IMM8|IMM32);
 		if (isXbyak_) {
 			puts("push(word, 1000);dump();");
 		} else {
@@ -1171,9 +1174,11 @@ class Test {
 		put("push", REG16|MEM16);
 		put("pop", REG16|MEM16);
 #ifdef XBYAK64
-		put("push", REG64|IMM32|MEM64);
+		put("push", IMM8);
+		put("push", REG64|MEM64);
 		put("pop", REG64|MEM64);
 #else
+		put("push", IMM8|IMM32);
 		put("push", REG32|IMM32|MEM32);
 		put("pop", REG32|MEM32);
 #endif
@@ -1270,9 +1275,9 @@ class Test {
 		put("mov", REG64, "0xffffffff12345678LL", "0xffffffff12345678");
 		put("mov", REG32e|REG16|REG8|RAX|EAX|AX|AL, IMM);
 
-		put("mov", EAX, "ptr[(void*)-1]", "[-1]");
-		put("mov", EAX, "ptr[(void*)0x7fffffff]", "[0x7fffffff]");
-		put("mov", EAX, "ptr[(void*)0xffffffffffffffff]", "[0xffffffffffffffff]");
+		put("mov", EAX, "ptr[(void*)-1]", "[abs -1]");
+		put("mov", EAX, "ptr[(void*)0x7fffffff]", "[abs 0x7fffffff]");
+		put("mov", EAX, "ptr[(void*)0xffffffffffffffff]", "[abs 0xffffffffffffffff]");
 	}
 	void putEtc() const
 	{
@@ -1293,9 +1298,9 @@ class Test {
 			put(p, REG64, "0x1234567890abcdefLL", "0x1234567890abcdef");
 			put("movbe", REG16|REG32e, MEM);
 			put("movbe", MEM, REG16|REG32e);
-#ifdef XBYAK64
-			put(p, RAX|EAX|AX|AL, "ptr [0x1234567890abcdefLL]", "[qword 0x1234567890abcdef]");
-			put(p, "ptr [0x1234567890abcdefLL]", "[qword 0x1234567890abcdef]", RAX|EAX|AX|AL);
+#if defined(XBYAK64) && !defined(__ILP32__)
+			put(p, RAX|EAX|AX|AL, "ptr [0x1234567890abcdefLL]", "[abs qword 0x1234567890abcdef]");
+			put(p, "ptr [0x1234567890abcdefLL]", "[abs qword 0x1234567890abcdef]", RAX|EAX|AX|AL);
 			put(p, "qword [rax], 0");
 			put(p, "qword [rax], 0x12");
 			put(p, "qword [rax], 0x1234");
@@ -1306,25 +1311,17 @@ class Test {
 #endif
 			put("mov", EAX, "ptr [eax + ecx * 0]", "[eax + ecx * 0]"); // ignore scale = 0
 		}
-		{
-			const char tbl[][8] = {
-				"movsx",
-				"movzx",
-			};
-			for (size_t i = 0; i < NUM_OF_ARRAY(tbl); i++) {
-				const char *p = tbl[i];
-				put(p, REG64, REG16|REG8|MEM8|MEM16);
-				put(p, REG32, REG16|REG8|MEM8|MEM16);
-				put(p, REG16, REG8|MEM8);
-			}
-		}
-#ifdef XBYAK64
-		put("movsxd", REG64, REG32|MEM32);
-#endif
 		put("cmpxchg8b", MEM);
 #ifdef XBYAK64
+		put("xchg", RAX, RAX);
 		put("cmpxchg16b", MEM);
+		put("cmpxchg16b", "xword [rax]", "oword [rax]");
 		put("fxrstor64", MEM);
+		put("xbegin", "0x12345678");
+		put("rdfsbase", REG32|REG64);
+		put("rdgsbase", REG32|REG64);
+		put("wrfsbase", REG32|REG64);
+		put("wrgsbase", REG32|REG64);
 #endif
 		{
 			const char tbl[][8] = {
@@ -1340,13 +1337,10 @@ class Test {
 			}
 		}
 
-		put("xchg", AL|REG8, AL|REG8|MEM);
 		put("xchg", MEM, AL|REG8);
-		put("xchg", AX|REG16, AX|REG16|MEM);
 		put("xchg", MEM, AX|REG16);
-		put("xchg", EAX|REG32, EAX|REG32|MEM);
 		put("xchg", MEM, EAX|REG32);
-		put("xchg", REG64, REG64|MEM);
+		put("xabort", IMM8);
 	}
 	void putShift() const
 	{
@@ -1492,18 +1486,6 @@ class Test {
 				put(p, XMM, XMM|MEM, IMM);
 			}
 		}
-		{
-			const char tbl[][16] = {
-				"pclmullqlqdq",
-				"pclmulhqlqdq",
-//				"pclmullqhdq", // QQQ : not supported by nasm/yasm
-//				"pclmulhqhdq",
-			};
-			for (size_t i = 0; i < NUM_OF_ARRAY(tbl); i++) {
-				const char *p = tbl[i];
-				put(p, XMM, XMM|MEM);
-			}
-		}
 		put("extractps", REG32e|MEM, XMM, IMM);
 		put("pextrw", REG32e|MEM, XMM, IMM); // pextrw for REG32 is for MMX2
 		put("pextrb", REG32e|MEM, XMM, IMM);
@@ -1519,6 +1501,24 @@ class Test {
 		put("pextrq", REG64|MEM, XMM, IMM);
 		put("pinsrq", XMM, REG64|MEM, IMM);
 #endif
+
+	}
+	void putVpclmulqdq()
+	{
+		const char tbl[][16] = {
+			"vpclmullqlqdq",
+			"vpclmulhqlqdq",
+			"vpclmullqhqdq",
+			"vpclmulhqhqdq",
+		};
+		for (size_t i = 0; i < NUM_OF_ARRAY(tbl); i++) {
+			const char *p = tbl[i] + 1; // remove the top 'v'
+			put(p, XMM, XMM|MEM);
+			p = tbl[i]; // use the top 'v'
+			put(p, XMM, XMM, XMM|MEM);
+			put(p, YMM, YMM, YMM|MEM);
+			put(p, ZMM, ZMM, ZMM|MEM);
+		}
 	}
 	void putSHA() const
 	{
@@ -2199,6 +2199,7 @@ class Test {
 			put("vcvtpd2ps", XMM, XMM | YMM | MEM);
 			put("vcvtpd2dq", XMM, XMM | YMM | MEM);
 			put("vcvttpd2dq", XMM, XMM | YMM | MEM);
+			put("vcvttpd2dq", YMM, MEM | ZMM_SAE);
 
 			put("vcvtph2ps", XMM | YMM, XMM | MEM);
 			put("vcvtps2ph", XMM | MEM, XMM | YMM, IMM8);
@@ -2511,8 +2512,40 @@ public:
 			}
 		}
 	}
+	// NASM 3 changes some encoding of movsx, xchg, so test them by YASM
+	void putNASMtoYASM()
+	{
+		{
+			const char tbl[][8] = {
+				"movsx",
+				"movzx",
+			};
+			for (size_t i = 0; i < NUM_OF_ARRAY(tbl); i++) {
+				const char *p = tbl[i];
+				put(p, REG64, REG16|REG8|MEM8|MEM16);
+				put(p, REG32, REG16|REG8|MEM8|MEM16);
+				put(p, REG16, REG8|MEM8);
+				put(p, "eax, ah");
+			}
+		}
+#ifdef XBYAK64
+		put("movsxd", REG64, REG32|MEM32);
+#endif
+		put("xchg", REG8, REG8|MEM);
+		put("xchg", REG16, REG16|MEM);
+		put("xchg", REG32, REG32|MEM);
+		put("xchg", _REG64|_REG64_2, REG64|MEM);
+		put("xchg", "cl, bl");
+		put("xchg", "cl, al");
+		put("xchg", "al, bl");
+		put("xchg", "di, si");
+		put("xchg", "ecx, ebp");
+	}
 	void put()
 	{
+#ifdef USE_YASM
+		putNASMtoYASM();
+#endif
 #ifdef USE_AVX512
 		putAVX512();
 #else
@@ -2567,6 +2600,7 @@ public:
 		putPushPop8_16();
 #else
 		putSIMPLE();
+		putVpclmulqdq();
 		putReg1();
 		putBt();
 		putRorM();
@@ -2605,7 +2639,7 @@ public:
 		putMPX();
 #endif
 
-#ifdef XBYAK64
+#if defined(XBYAK64) && !defined(__ILP32__)
 
 #ifdef USE_YASM
 		putRip();
@@ -3367,6 +3401,8 @@ public:
 				}
 			}
 		}
+		put("vfmadd132pd", ZMM, ZMM, ZMM_ER);
+		put("vfmadd132ps", ZMM, ZMM, ZMM_ER);
 	}
 	void put512_Y_XM()
 	{
@@ -3440,6 +3476,11 @@ public:
 				put(p, _ZMM, _ZMM, mem);
 			}
 		}
+		put("vaddpd", ZMM, ZMM, ZMM_ER);
+		put("vmaxpd", ZMM, ZMM, ZMM_SAE);
+		put("vminps", ZMM, ZMM, ZMM_SAE);
+		put("vmaxsd", XMM, XMM, XMM_SAE);
+		put("vminss", XMM, XMM, XMM_SAE);
 #endif
 	}
 	void put512_cvt()
@@ -3457,6 +3498,10 @@ public:
 		put("vcvtpd2dq", YMM_KZ, _ZMM | ZMM_ER | M_1to8);
 #endif
 	}
+	void put512_fp16()
+	{
+		put("vaddph", ZMM, ZMM, ZMM_ER);
+	}
 	void putMin()
 	{
 #ifdef XBYAK64
@@ -3496,6 +3541,7 @@ public:
 		put512_AVX1();
 		separateFunc();
 		put512_cvt();
+		put512_fp16();
 #endif
 	}
 #endif
