@@ -5,6 +5,7 @@ set -e
 FILTER="grep -v warning"
 CXX=${CXX:=g++}
 CFLAGS_USER=${CFLAGS}
+CFLAGS_WARN="$(cat CFLAGS_WARN.cfg)"
 
 case $1 in
 64)
@@ -22,7 +23,7 @@ case $1 in
 	;;
 esac
 
-CFLAGS="$CFLAGS_USER -Wall -I../ $OPT2 -DUSE_AVX512"
+CFLAGS="$CFLAGS_USER $CFLAGS_WARN -I../ $OPT2 -DUSE_AVX512"
 echo "compile make_512.cpp"
 $CXX $CFLAGS make_512.cpp -o make_512
 
