@@ -4,6 +4,7 @@ set -e
 XED=${XED:=xed}
 CXX=${CXX:=g++}
 CFLAGS_USER=${CFLAGS}
+CFLAGS_WARN="$(cat CFLAGS_WARN.cfg)"
 PYTHON=${PYTHON:=python3}
 echo $XED
 
@@ -14,7 +15,7 @@ fi
 
 TARGET=$1
 
-CFLAGS="$CFLAGS_USER -g -Wall -Wextra -I ../"
+CFLAGS="$CFLAGS_USER $CFLAGS_WARN -I ../"
 
 echo "test:" $TARGET
 cp $TARGET tmp.cpp
