@@ -5,6 +5,7 @@ set -e
 FILTER=cat
 CXX=${CXX:=g++}
 CFLAGS_USER=${CFLAGS}
+CFLAGS_WARN="$(cat CFLAGS_WARN.cfg)"
 
 case $1 in
 Y)
@@ -48,7 +49,7 @@ noexcept)
 	;;
 esac
 
-CFLAGS="$CFLAGS_USER -Wall -g -I../ -I./ $OPT2"
+CFLAGS="$CFLAGS_USER $CFLAGS_WARN -g -I../ -I./ $OPT2"
 echo "compile make_nm.cpp with $CFLAGS"
 $CXX $CFLAGS make_nm.cpp -o make_nm
 
